@@ -16,24 +16,21 @@
 # specific language governing permissions and limitations
 # under the License.
 """Hook for SSH connections."""
+from __future__ import annotations
+
 import os
-import sys
 import warnings
 from base64 import decodebytes
 from io import StringIO
 from select import select
-from typing import Any, Dict, Optional, Sequence, Tuple, Type, Union
+from typing import Any, Sequence
 
 import paramiko
 from paramiko.config import SSH_PORT
 from sshtunnel import SSHTunnelForwarder
 from tenacity import Retrying, stop_after_attempt, wait_fixed, wait_random
 
-if sys.version_info >= (3, 8):
-    from functools import cached_property
-else:
-    from cached_property import cached_property
-
+from airflow.compat.functools import cached_property
 from airflow.exceptions import AirflowException
 from airflow.hooks.base import BaseHook
 
@@ -71,10 +68,11 @@ class SSHHook(BaseHook):
     :param disabled_algorithms: dictionary mapping algorithm type to an
         iterable of algorithm identifiers, which will be disabled for the
         lifetime of the transport
+    :param ciphers: list of ciphers to use in order of preference
     """
 
     # List of classes to try loading private keys as, ordered (roughly) by most common to least common
-    _pkey_loaders: Sequence[Type[paramiko.PKey]] = (
+    _pkey_loaders: Sequence[type[paramiko.PKey]] = (
         paramiko.RSAKey,
         paramiko.ECDSAKey,
         paramiko.Ed25519Key,
@@ -94,7 +92,7 @@ class SSHHook(BaseHook):
     hook_name = 'SSH'
 
     @staticmethod
-    def get_ui_field_behaviour() -> Dict[str, Any]:
+    def get_ui_field_behaviour() -> dict[str, Any]:
         """Returns custom field behaviour"""
         return {
             "hidden_fields": ['schema'],
@@ -105,17 +103,18 @@ class SSHHook(BaseHook):
 
     def __init__(
         self,
-        ssh_conn_id: Optional[str] = None,
+        ssh_conn_id: str | None = None,
         remote_host: str = '',
-        username: Optional[str] = None,
-        password: Optional[str] = None,
-        key_file: Optional[str] = None,
-        port: Optional[int] = None,
-        timeout: Optional[int] = None,
-        conn_timeout: Optional[int] = None,
+        username: str | None = None,
+        password: str | None = None,
+        key_file: str | None = None,
+        port: int | None = None,
+        timeout: int | None = None,
+        conn_timeout: int | None = None,
         keepalive_interval: int = 30,
         banner_timeout: float = 30.0,
-        disabled_algorithms: Optional[dict] = None,
+        disabled_algorithms: dict | None = None,
+        ciphers: list[str] | None = None,
     ) -> None:
         super().__init__()
         self.ssh_conn_id = ssh_conn_id
@@ -130,6 +129,7 @@ class SSHHook(BaseHook):
         self.keepalive_interval = keepalive_interval
         self.banner_timeout = banner_timeout
         self.disabled_algorithms = disabled_algorithms
+        self.ciphers = ciphers
         self.host_proxy_cmd = None
 
         # Default values, overridable from Connection
@@ -140,7 +140,7 @@ class SSHHook(BaseHook):
         self.look_for_keys = True
 
         # Placeholder for deprecated __enter__
-        self.client: Optional[paramiko.SSHClient] = None
+        self.client: paramiko.SSHClient | None = None
 
         # Use connection to override defaults
         if self.ssh_conn_id is not None:
@@ -205,6 +205,9 @@ class SSHHook(BaseHook):
                 if "disabled_algorithms" in extra_options:
                     self.disabled_algorithms = extra_options.get("disabled_algorithms")
 
+                if "ciphers" in extra_options:
+                    self.ciphers = extra_options.get("ciphers")
+
                 if host_key is not None:
                     if host_key.startswith("ssh-"):
                         key_type, host_key = host_key.split(None)[:2]
@@ -261,7 +264,7 @@ class SSHHook(BaseHook):
         self.port = self.port or SSH_PORT
 
     @cached_property
-    def host_proxy(self) -> Optional[paramiko.ProxyCommand]:
+    def host_proxy(self) -> paramiko.ProxyCommand | None:
         cmd = self.host_proxy_cmd
         return paramiko.ProxyCommand(cmd) if cmd else None
 
@@ -279,11 +282,18 @@ class SSHHook(BaseHook):
                 "Remote Identification Change is not verified. "
                 "This won't protect against Man-In-The-Middle attacks"
             )
+            # to avoid BadHostKeyException, skip loading host keys
+            client.set_missing_host_key_policy(paramiko.MissingHostKeyPolicy)
         else:
             client.load_system_host_keys()
 
         if self.no_host_key_check:
             self.log.warning("No Host Key Verification. This won't protect against Man-In-The-Middle attacks")
+            client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
+            # to avoid BadHostKeyException, skip loading and saving host keys
+            known_hosts = os.path.expanduser("~/.ssh/known_hosts")
+            if not self.allow_host_key_change and os.path.isfile(known_hosts):
+                client.load_host_keys(known_hosts)
         else:
             if self.host_key is not None:
                 client_host_keys = client.get_host_keys()
@@ -296,11 +306,7 @@ class SSHHook(BaseHook):
             else:
                 pass  # will fallback to system host keys if none explicitly specified in conn extra
 
-        if self.no_host_key_check or self.allow_host_key_change:
-            # Default is RejectPolicy
-            client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
-
-        connect_kwargs: Dict[str, Any] = dict(
+        connect_kwargs: dict[str, Any] = dict(
             hostname=self.remote_host,
             username=self.username,
             timeout=self.conn_timeout,
@@ -342,10 +348,15 @@ class SSHHook(BaseHook):
             # type "Optional[Transport]" and item "None" has no attribute "set_keepalive".
             client.get_transport().set_keepalive(self.keepalive_interval)  # type: ignore[union-attr]
 
+        if self.ciphers:
+            # MyPy check ignored because "paramiko" isn't well-typed. The `client.get_transport()` returns
+            # type "Optional[Transport]" and item "None" has no method `get_security_options`".
+            client.get_transport().get_security_options().ciphers = self.ciphers  # type: ignore[union-attr]
+
         self.client = client
         return client
 
-    def __enter__(self) -> 'SSHHook':
+    def __enter__(self) -> SSHHook:
         warnings.warn(
             'The contextmanager of SSHHook is deprecated.'
             'Please use get_conn() as a contextmanager instead.'
@@ -360,7 +371,7 @@ class SSHHook(BaseHook):
             self.client = None
 
     def get_tunnel(
-        self, remote_port: int, remote_host: str = "localhost", local_port: Optional[int] = None
+        self, remote_port: int, remote_host: str = "localhost", local_port: int | None = None
     ) -> SSHTunnelForwarder:
         """
         Creates a tunnel between two hosts. Like ssh -L <LOCAL_PORT>:host:<REMOTE_PORT>.
@@ -372,7 +383,7 @@ class SSHHook(BaseHook):
         :return: sshtunnel.SSHTunnelForwarder object
         """
         if local_port:
-            local_bind_address: Union[Tuple[str, int], Tuple[str]] = ('localhost', local_port)
+            local_bind_address: tuple[str, int] | tuple[str] = ('localhost', local_port)
         else:
             local_bind_address = ('localhost',)
 
@@ -421,7 +432,7 @@ class SSHHook(BaseHook):
 
         return self.get_tunnel(remote_port, remote_host, local_port)
 
-    def _pkey_from_private_key(self, private_key: str, passphrase: Optional[str] = None) -> paramiko.PKey:
+    def _pkey_from_private_key(self, private_key: str, passphrase: str | None = None) -> paramiko.PKey:
         """
         Creates appropriate paramiko key for given private key
 
@@ -452,9 +463,9 @@ class SSHHook(BaseHook):
         ssh_client: paramiko.SSHClient,
         command: str,
         get_pty: bool,
-        environment: Optional[dict],
-        timeout: Optional[int],
-    ) -> Tuple[int, bytes, bytes]:
+        environment: dict | None,
+        timeout: int | None,
+    ) -> tuple[int, bytes, bytes]:
         self.log.info("Running command: %s", command)
 
         # set timeout taken as params
